<?php
/* --------------------------------------------------------------
   CustomerGroupReader.inc.php 2017-09-011
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class CustomerGroupReader
 *
 * @category   System
 * @package    CustomerGroup
 * @subpackage Repositories
 */
class CustomerGroupReader implements CustomerGroupReaderInterface
{
	/**
	 * @var \CI_DB_query_builder
	 */
	protected $queryBuilder;
	
	
	public function __construct(CI_DB_query_builder $queryBuilder)
	{
		$this->queryBuilder = $queryBuilder;
	}
	
	
	/**
	 * Returns all customer group entities data as array.
	 *
	 * @return array
	 */
	public function getAll()
	{
		$rawData = $this->queryBuilder->from('customers_status')
		                              ->order_by('customers_status_id')
		                              ->get()
		                              ->result_array() ?: [];
		$resultData     = [];
		$customerGroups = [];
		
		foreach($rawData as $data)
		{
			if(isset($oldId) && $oldId !== (int)$data['customers_status_id'])
			{
				$resultData[]   = $customerGroups;
				$customerGroups = [];
			}
			$oldId = (int)$data['customers_status_id'];
			
			$customerGroups['id']                          = $oldId;
			$customerGroups['names'][$data['language_id']] = $data['customers_status_name'];
			
			$customerGroups['settings']       = $this->_getSettingsData($data);
			$customerGroups['configurations'] = $this->_getConfigurationsData($data);
		}
		if(count($rawData) > 0)
		{
			$resultData[] = $customerGroups;
		}
		
		return $resultData;
	}
	
	
	/**
	 * Returns customer group entity data by the given id.
	 *
	 * @param \IntType $id
	 *
	 * @throws \EntityNotFoundException
	 *
	 * @return array
	 */
	public function getById(IntType $id)
	{
		$rawData = $this->queryBuilder->select()
		                              ->from('customers_status')
		                              ->where('customers_status_id', $id->asInt())
		                              ->get()
		                              ->result_array() ? : [];
		
		if(count($rawData) === 0)
		{
			throw new EntityNotFoundException('Customer group entity was not found with provided id "' . $id->asInt()
			                                  . '"');
		}
		
		$result              = ['id' => $id->asInt()];
		$customerGroupsNames = [];
		
		foreach($rawData as $data)
		{
			$customerGroupsNames[$data['language_id']] = $data['customers_status_name'];
			$result['settings']                        = $this->_getSettingsData($data);
			$result['configurations']                  = $this->_getConfigurationsData($data);
		}
		$result['names'] = $customerGroupsNames;
		
		return $result;
	}
	
	
	private function _getSettingsData(array $customerGroupData)
	{
		return [
			'public'             => (int)$customerGroupData['customers_status_public'] === 1,
			'otDiscount'         => (int)$customerGroupData['customers_status_ot_discount_flag'] === 1,
			'graduatedPrices'    => (int)$customerGroupData['customers_status_graduated_prices'] === 1,
			'showPrice'          => (int)$customerGroupData['customers_status_show_price'] === 1,
			'showPriceTax'       => (int)$customerGroupData['customers_status_show_price_tax'] === 1,
			'addTaxOt'           => (int)$customerGroupData['customers_status_add_tax_ot'] === 1,
			'discountAttributes' => (int)$customerGroupData['customers_status_discount_attributes'] === 1,
			'fsk18'              => (int)$customerGroupData['customers_fsk18'] === 1,
			'fsk18Display'       => (int)$customerGroupData['customers_fsk18_display'] === 1,
			'writeReviews'       => (int)$customerGroupData['customers_status_write_reviews'] === 1,
			'readReviews'        => (int)$customerGroupData['customers_status_read_reviews'] === 1,
		];
	}
	
	
	private function _getConfigurationsData(array $customerGroupData)
	{
		return [
			'minOrder'                 => $customerGroupData['customers_status_min_order'] ? (double)$customerGroupData['customers_status_min_order'] : null,
			'maxOrder'                 => $customerGroupData['customers_status_max_order'] ? (double)$customerGroupData['customers_status_max_order'] : null,
			'discount'                 => $customerGroupData['customers_status_discount'] ? (double)$customerGroupData['customers_status_discount'] : null,
			'otDiscount'               => $customerGroupData['customers_status_ot_discount'] ? (double)$customerGroupData['customers_status_ot_discount'] : null,
			'unallowedPaymentModules'  => $customerGroupData['customers_status_payment_unallowed'] !== '' ? explode(',',
			                                                                                                        $customerGroupData['customers_status_payment_unallowed']) : [],
			'unallowedShippingModules' => $customerGroupData['customers_status_payment_unallowed'] !== '' ? explode(',',
			                                                                                                        $customerGroupData['customers_status_shipping_unallowed']) : [],
		];
	}
}